use reqwest::Client;
use futures_util::StreamExt;
use std::io;

#[derive(Debug)]
pub enum Error {
    Reqwest (reqwest::Error),
    Io (io::Error),
    JSON (serde_json::Error),
}

impl From<serde_json::Error> for Error {
    fn from(value: serde_json::Error) -> Self {
        Self::JSON(value)
    }
}

impl From<reqwest::Error> for Error {
    fn from(value: reqwest::Error) -> Self {
        Self::Reqwest(value)
    }
}

impl From<io::Error> for Error {
    fn from(value: io::Error) -> Self {
        Self::Io(value)
    }
}

pub async fn start_stream(api_endpoint: String, name: String, token: String, resource_types: Vec<String> ) -> Result<(), Error> {

    let req = crate::ResourceRequest {
        request_origin: name,
        resource_types: resource_types,
    };
    let json = serde_json::to_string(&req)?;

    let auth_value = format!("Bearer {}",token);

    let client = Client::new();
    let mut stream = client.get(api_endpoint)
        .header("Authorization", &auth_value)
        .body(json)
        .send()
        .await?
        .bytes_stream();

    while let Some(chunk) = stream.next().await {
        println!("Chunk: {:?}", chunk?);
    };
    Ok(())
}
